##############################################################################
#
# Copyright (c) 2003 - Tue Wennerberg - All Rights Reserved
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from Products.PageTemplates.PageTemplateFile import PageTemplateFile
import OFS.ObjectManager
import OFS.PropertyManager
import OFS.SimpleItem
import OFS.Folder
import Globals
import Acquisition
import AccessControl
import AccessControl.Role
import App.Management
from App.FindHomes import SOFTWARE_HOME
from Products.PythonScripts.PythonScript import PythonScript
from Products.PageTemplates.ZopePageTemplate import manage_addPageTemplate
from Products.ZMySQLDA.DA import manage_addZMySQLConnection
from Products.ZSQLMethods.SQL import manage_addZSQLMethod

################################################################################

manage_addRootForm=PageTemplateFile('Products/ZTimeReg/zpt/addRootForm.zpt')

def manage_addRoot(self, id, title='',
                   db_conn_id='',
                   install_style='',
                   regs_table='',
                   create_table='',
                   REQUEST=None):
    """Add an instance
    """
    id = str(id)
    title = str(title)
    templates = []

    # Create new object instance
    if install_style=='cmf':
        self.manage_addPortalFolder(id, title)
    elif install_style=='plone':
        self.manage_addPloneFolder(id, title)
    else:
        OFS.Folder.manage_addFolder(self, id, title)
        templates.append('main_template')

    newobj = self._getOb(id)

    # Restrict permission
    newobj.manage_permission('View',
                             roles=['Authenticated'],
                             acquire=0);

    # Add scripts to root folder
    scripts = ['calculateAccounts', 'decode_time', 'encode_time',
               'getAccount', 'getCustomer', 'getFiltering',
               'getUser', 'getUsername', 'insertRegistration',
               'isCustomer', 'round']
    for script_id in scripts:
        filename = SOFTWARE_HOME + '/Products/ZTimeReg/scripts/' + script_id + '.py'
        scriptobj = PythonScript(script_id)
        sourcecode = open(filename).read()
        scriptobj.write(sourcecode)
        newobj._setObject(script_id, scriptobj)

    # Add page templates to root folder
    templates = templates + ['accounts_html', 'cust_index_html', 'index_html'
                             , 'register', 'registrations_html']
    for template_id in templates:
        filename = SOFTWARE_HOME + '/Products/ZTimeReg/zpt/' + template_id + '.zpt'
        text = open(filename).read()
        manage_addPageTemplate(newobj, template_id, text=text)

    # Set regs table name
    newobj.manage_addProperty('regs_table',regs_table,'string')

    # Add SQLs
    sql_names = ['insertRegistration', 'lookupAccounts', 'lookupExistingRegs',
                 'createTable', 'lookupRegs']
    for sql_name in sql_names:
        filename = SOFTWARE_HOME + '/Products/ZTimeReg/sql/' + sql_name + '.sql'
        sql_id = sql_name + '_sql'
        file = open(filename)
        arguments = file.readline()
        sql = file.read()
        manage_addZSQLMethod(newobj, sql_id, '', db_conn_id, arguments, sql)
    
    # Create table?
    if create_table:
        create_obj = newobj._getOb('createTable_sql')
        create_obj()
        #filename = SOFTWARE_HOME + '/Products/ZTimeReg/sql/schema.sql'
        #sql = open(filename).read()
        #db_conn = newobj._getOb(db_conn_id)
        #db_conn.manage_test(sql)

    # Redirect to management screen?
    if REQUEST is not None:
        return self.manage_main(self,REQUEST)

################################################################################

class Root( OFS.ObjectManager.ObjectManager,
            OFS.PropertyManager.PropertyManager,
            OFS.SimpleItem.Item,
            Globals.Persistent,
            Acquisition.Implicit,
            AccessControl.Role.RoleManager,
            App.Management.Navigation ):
    
    # This is a dummy class which is never instantiated.
    
    meta_type = 'TimeReg System'
    
    ZopeTime = Acquisition.Acquired
    HelpSys  = Acquisition.Acquired
    security = AccessControl.ClassSecurityInfo()
        
    def __init__(self, id, title):
        "Init"
        self.id = id
        self.manage_edit(title)
        
    _properties = (
        {'id':'title', 'type':'string', 'mode':'w'},
        )
    
    manage_options = (
        OFS.ObjectManager.ObjectManager.manage_options +
        OFS.PropertyManager.PropertyManager.manage_options +
        OFS.SimpleItem.Item.manage_options +
        AccessControl.Role.RoleManager.manage_options
        )

    def manage_edit(self, title):
        "Edit"
        self.title = title
        
